window.Gleam.initSearch([{"doc":"p5js_gleam","title":"p5js_gleam","content":"# p5js_gleam\r\n\r\n[![Package Version](https://img.shields.io/hexpm/v/p5js_gleam)](https://hex.pm/packages/p5js_gleam)\r\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/p5js_gleam/)\r\n\r\nA simple game library providing [p5js](https://p5js.org/) bindings for Gleam in a functional style to make basic games and animations. Heavily inspired by the Racket library 2htdp/universe.\r\n\r\nThis project is primarily made up of\r\n\r\n- A [bun](https://bun.sh/) script that is used to generate bindings for p5js class methods\r\n  - Script is based on script created for [glare](https://github.com/Enderchief/glare/tree/master/scripts)\r\n- A wrapper game engine that provides a functional interface to p5js\r\n\r\n## Showcase\r\n\r\nAn example generated maze\r\n\r\n![maze_room](https://github.com/Acepie/p5js_gleam/assets/5996838/e7e21eb0-dad2-486d-99bb-40f4c088e272)\r\n\r\nAn example ball spawner\r\n\r\n![ball_spawner](https://github.com/Acepie/p5js_gleam/assets/5996838/4869513b-095a-4244-8fc6-6f63738e4c97)\r\n\r\n## Using the library\r\n\r\nAdd the library to your project by running the following command\r\n\r\n```bash\r\ngleam add p5js_gleam\r\n```\r\n\r\nThen add a call to the `start_sketch` function in your main function.\r\n\r\n```gleam\r\nimport p5js_gleam.{type P5}\r\nimport p5js_gleam/bindings as p5\r\n\r\nfn setup(p: P5) -&gt; String {\r\n  p5.create_canvas(p, 800.0, 600.0)\r\n  &quot;Hello, world!&quot;\r\n}\r\n\r\nfn draw(p: P5, state: String) {\r\n  p5.background(p, &quot;#ffffff&quot;)\r\n  p5.fill(p, &quot;#000000&quot;)\r\n  p5.text(p, state, 400.0, 300.0)\r\n}\r\n\r\npub fn main() {\r\n  p5js_gleam.create_sketch(init: setup, draw: draw)\r\n  |&gt; p5.start_sketch\r\n}\r\n```\r\n\r\nAfterwards you will need to build your project and include it in an html file. The html file must import p5js as a module before the generated javascript file. The easiest way to do this is to load the p5js library from a CDN.\r\n\r\n```html\r\n&lt;!doctype html&gt;\r\n&lt;html lang=&quot;en&quot;&gt;\r\n  &lt;head&gt;\r\n    &lt;meta charset=&quot;UTF-8&quot; /&gt;\r\n    &lt;meta http-equiv=&quot;X-UA-Compatible&quot; content=&quot;IE=edge&quot; /&gt;\r\n    &lt;meta name=&quot;viewport&quot; content=&quot;width=device-width, initial-scale=1.0&quot; /&gt;\r\n    &lt;title&gt;Ball Spawner&lt;/title&gt;\r\n    &lt;script src=&quot;https://cdn.jsdelivr.net/npm/p5@1.9.2/lib/p5.js&quot;&gt;&lt;/script&gt;\r\n    &lt;script type=&quot;module&quot;&gt;\r\n      import { main } from &quot;./ball_spawner.js&quot;;\r\n      main();\r\n    &lt;/script&gt;\r\n  &lt;/head&gt;\r\n  &lt;body&gt;&lt;/body&gt;\r\n&lt;/html&gt;\r\n```\r\n\r\nTo see the library in action you can serve one of the examples in the `examples` directory.\r\n\r\n## Generating FFI Bindings\r\n\r\nTo generate the FFI bindings you will need bun installed. Once you have bun installed run\r\n\r\n```bash\r\nbun run ./scripts/generate_p5.ts\r\n```\r\n","url":"index.html"},{"doc":"p5js_gleam","title":"Assets","content":"pub opaque type Assets\n Assets that are loaded in `preload` like images and fonts. Can be used later during rendering.\n\nSynonyms:\np5js_gleam.Assets\np5js_gleam Assets","url":"p5js_gleam.html#Assets"},{"doc":"p5js_gleam","title":"P5","content":"pub type P5\n A reference to the P5js library. Primarily used to access the P5js APIs for drawing.\n\nSynonyms:\np5js_gleam.P5\np5js_gleam P5","url":"p5js_gleam.html#P5"},{"doc":"p5js_gleam","title":"P5Font","content":"pub type P5Font\n A reference to a P5js font object. Should be used by passing it to the P5js `textFont` function.\n\nSynonyms:\np5js_gleam.P5Font\np5js_gleam P5Font","url":"p5js_gleam.html#P5Font"},{"doc":"p5js_gleam","title":"P5Image","content":"pub type P5Image\n A reference to a P5js image object. Should be used by passing it to the P5js `image` function.\n\nSynonyms:\np5js_gleam.P5Image\np5js_gleam P5Image","url":"p5js_gleam.html#P5Image"},{"doc":"p5js_gleam","title":"SketchConfig","content":"pub opaque type SketchConfig(model, ignored)\n Configuration to start a sketch. Requires an init function to create the initial model, and a draw function to draw the model. Other handlers can also be provided to update the model based on different user events. Use the `create_sketch` function and the `set_*` helpers to initialize your config.\n\nSynonyms:\np5js_gleam.SketchConfig\np5js_gleam SketchConfig","url":"p5js_gleam.html#SketchConfig"},{"doc":"p5js_gleam","title":"create_sketch","content":"pub fn create_sketch(\n  init init: fn(P5) -&gt; a,\n  draw draw: fn(P5, a) -&gt; b,\n) -&gt; SketchConfig(a, b)\n Creates a minimal sketch configuration.\nSynonyms:\np5js_gleam.create_sketch\np5js_gleam create_sketch","url":"p5js_gleam.html#create_sketch"},{"doc":"p5js_gleam","title":"create_sketch_with_preloading","content":"pub fn create_sketch_with_preloading(\n  preload preload: fn(P5) -&gt; Assets,\n  init init: fn(P5) -&gt; a,\n  draw draw: fn(P5, a, Assets) -&gt; b,\n) -&gt; SketchConfig(a, b)\n Creates a sketch configuration that supports preloading assets.\nSynonyms:\np5js_gleam.create_sketch_with_preloading\np5js_gleam create_sketch_with_preloading","url":"p5js_gleam.html#create_sketch_with_preloading"},{"doc":"p5js_gleam","title":"get_font","content":"pub fn get_font(\n  assets: Assets,\n  font_key: String,\n) -&gt; Result(P5Font, Nil)\n Get a font from the asset set.\nSynonyms:\np5js_gleam.get_font\np5js_gleam get_font","url":"p5js_gleam.html#get_font"},{"doc":"p5js_gleam","title":"get_image","content":"pub fn get_image(\n  assets: Assets,\n  image_key: String,\n) -&gt; Result(P5Image, Nil)\n Get an image from the asset set.\nSynonyms:\np5js_gleam.get_image\np5js_gleam get_image","url":"p5js_gleam.html#get_image"},{"doc":"p5js_gleam","title":"initialize_assets","content":"pub fn initialize_assets() -&gt; Assets\n Creates an initial empty asset set.\nSynonyms:\np5js_gleam.initialize_assets\np5js_gleam initialize_assets","url":"p5js_gleam.html#initialize_assets"},{"doc":"p5js_gleam","title":"insert_font","content":"pub fn insert_font(\n  assets: Assets,\n  font_key: String,\n  font: P5Font,\n) -&gt; Assets\n Adds a font to the asset set.\nSynonyms:\np5js_gleam.insert_font\np5js_gleam insert_font","url":"p5js_gleam.html#insert_font"},{"doc":"p5js_gleam","title":"insert_image","content":"pub fn insert_image(\n  assets: Assets,\n  image_key: String,\n  image: P5Image,\n) -&gt; Assets\n Adds an image to the asset set.\nSynonyms:\np5js_gleam.insert_image\np5js_gleam insert_image","url":"p5js_gleam.html#insert_image"},{"doc":"p5js_gleam","title":"set_on_key_pressed","content":"pub fn set_on_key_pressed(\n  s: SketchConfig(a, b),\n  on_key_pressed: fn(String, Int, a) -&gt; a,\n) -&gt; SketchConfig(a, b)\n Sets the `on_key_pressed` function to be called.\nSynonyms:\np5js_gleam.set_on_key_pressed\np5js_gleam set_on_key_pressed","url":"p5js_gleam.html#set_on_key_pressed"},{"doc":"p5js_gleam","title":"set_on_key_released","content":"pub fn set_on_key_released(\n  s: SketchConfig(a, b),\n  on_key_released: fn(String, Int, a) -&gt; a,\n) -&gt; SketchConfig(a, b)\n Sets the `on_key_released` function to be called.\nSynonyms:\np5js_gleam.set_on_key_released\np5js_gleam set_on_key_released","url":"p5js_gleam.html#set_on_key_released"},{"doc":"p5js_gleam","title":"set_on_mouse_clicked","content":"pub fn set_on_mouse_clicked(\n  s: SketchConfig(a, b),\n  on_mouse_clicked: fn(Float, Float, a) -&gt; a,\n) -&gt; SketchConfig(a, b)\n Sets the `on_mouse_clicked` function to be called.\nSynonyms:\np5js_gleam.set_on_mouse_clicked\np5js_gleam set_on_mouse_clicked","url":"p5js_gleam.html#set_on_mouse_clicked"},{"doc":"p5js_gleam","title":"set_on_mouse_moved","content":"pub fn set_on_mouse_moved(\n  s: SketchConfig(a, b),\n  on_mouse_moved: fn(Float, Float, a) -&gt; a,\n) -&gt; SketchConfig(a, b)\n Sets the `on_mouse_moved` function to be called.\nSynonyms:\np5js_gleam.set_on_mouse_moved\np5js_gleam set_on_mouse_moved","url":"p5js_gleam.html#set_on_mouse_moved"},{"doc":"p5js_gleam","title":"set_on_tick","content":"pub fn set_on_tick(\n  s: SketchConfig(a, b),\n  on_tick: fn(a) -&gt; a,\n) -&gt; SketchConfig(a, b)\n Sets the `on_tick` function to be called.\nSynonyms:\np5js_gleam.set_on_tick\np5js_gleam set_on_tick","url":"p5js_gleam.html#set_on_tick"},{"doc":"p5js_gleam","title":"p5js_gleam","content":"","url":"p5js_gleam.html"},{"doc":"p5js_gleam/bindings","title":"background","content":"pub fn background(p: P5, color: String) -&gt; P5\n A binding to the p5.js [`background`](https://p5js.org/reference/#/p5/background) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.background\np5js_gleam/bindings background","url":"p5js_gleam/bindings.html#background"},{"doc":"p5js_gleam/bindings","title":"circle","content":"pub fn circle(\n  p: P5,\n  x_center: Float,\n  y_center: Float,\n  diameter: Float,\n) -&gt; P5\n A binding to the p5.js [`circle`](https://p5js.org/reference/#/p5/circle) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.circle\np5js_gleam/bindings circle","url":"p5js_gleam/bindings.html#circle"},{"doc":"p5js_gleam/bindings","title":"create_canvas","content":"pub fn create_canvas(p: P5, width: Float, height: Float) -&gt; P5\n A binding to the p5.js [`createCanvas`](https://p5js.org/reference/#/p5/createCanvas) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.create_canvas\np5js_gleam/bindings create_canvas","url":"p5js_gleam/bindings.html#create_canvas"},{"doc":"p5js_gleam/bindings","title":"ellipse","content":"pub fn ellipse(\n  p: P5,\n  x_center: Float,\n  y_center: Float,\n  width: Float,\n  height: Float,\n) -&gt; P5\n A binding to the p5.js [`ellipse`](https://p5js.org/reference/#/p5/ellipse) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.ellipse\np5js_gleam/bindings ellipse","url":"p5js_gleam/bindings.html#ellipse"},{"doc":"p5js_gleam/bindings","title":"erase","content":"pub fn erase(p: P5, strength: Int, edge_strength: Int) -&gt; P5\n A binding to the p5.js [`erase`](https://p5js.org/reference/#/p5/erase) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.erase\np5js_gleam/bindings erase","url":"p5js_gleam/bindings.html#erase"},{"doc":"p5js_gleam/bindings","title":"fill","content":"pub fn fill(p: P5, color_hex: String) -&gt; P5\n A binding to the p5.js [`fill`](https://p5js.org/reference/#/p5/fill) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.fill\np5js_gleam/bindings fill","url":"p5js_gleam/bindings.html#fill"},{"doc":"p5js_gleam/bindings","title":"image","content":"pub fn image(\n  p: P5,\n  image: P5Image,\n  top_left_x: Float,\n  top_left_y: Float,\n  width: Float,\n  height: Float,\n) -&gt; P5\n A binding to the p5.js [`image`](https://p5js.org/reference/#/p5/image) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.image\np5js_gleam/bindings image","url":"p5js_gleam/bindings.html#image"},{"doc":"p5js_gleam/bindings","title":"line","content":"pub fn line(\n  p: P5,\n  point1_x: Float,\n  point1_y: Float,\n  point2_x: Float,\n  point2_y: Float,\n) -&gt; P5\n A binding to the p5.js [`line`](https://p5js.org/reference/#/p5/line) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.line\np5js_gleam/bindings line","url":"p5js_gleam/bindings.html#line"},{"doc":"p5js_gleam/bindings","title":"load_font","content":"pub fn load_font(p: P5, path: String) -&gt; P5Font\n A binding to the p5.js [`loadFont`](https://p5js.org/reference/#/p5/loadFont) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.load_font\np5js_gleam/bindings load_font","url":"p5js_gleam/bindings.html#load_font"},{"doc":"p5js_gleam/bindings","title":"load_image","content":"pub fn load_image(p: P5, path: String) -&gt; P5Image\n A binding to the p5.js [`loadImage`](https://p5js.org/reference/#/p5/loadImage) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.load_image\np5js_gleam/bindings load_image","url":"p5js_gleam/bindings.html#load_image"},{"doc":"p5js_gleam/bindings","title":"no_erase","content":"pub fn no_erase(p: P5) -&gt; P5\n A binding to the p5.js [`noErase`](https://p5js.org/reference/#/p5/noErase) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.no_erase\np5js_gleam/bindings no_erase","url":"p5js_gleam/bindings.html#no_erase"},{"doc":"p5js_gleam/bindings","title":"no_fill","content":"pub fn no_fill(p: P5) -&gt; P5\n A binding to the p5.js [`noFill`](https://p5js.org/reference/#/p5/noFill) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.no_fill\np5js_gleam/bindings no_fill","url":"p5js_gleam/bindings.html#no_fill"},{"doc":"p5js_gleam/bindings","title":"no_stroke","content":"pub fn no_stroke(p: P5) -&gt; P5\n A binding to the p5.js [`noStroke`](https://p5js.org/reference/#/p5/noStroke) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.no_stroke\np5js_gleam/bindings no_stroke","url":"p5js_gleam/bindings.html#no_stroke"},{"doc":"p5js_gleam/bindings","title":"quad","content":"pub fn quad(\n  p: P5,\n  p1_x: Float,\n  p1_y: Float,\n  p2_x: Float,\n  p2_y: Float,\n  p3_x: Float,\n  p3_y: Float,\n  p4_x: Float,\n  p4_y: Float,\n) -&gt; P5\n A binding to the p5.js [`quad`](https://p5js.org/reference/#/p5/quad) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.quad\np5js_gleam/bindings quad","url":"p5js_gleam/bindings.html#quad"},{"doc":"p5js_gleam/bindings","title":"rect","content":"pub fn rect(\n  p: P5,\n  top_left_x: Float,\n  top_left_y: Float,\n  width: Float,\n  height: Float,\n) -&gt; P5\n A binding to the p5.js [`rect`](https://p5js.org/reference/#/p5/rect) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.rect\np5js_gleam/bindings rect","url":"p5js_gleam/bindings.html#rect"},{"doc":"p5js_gleam/bindings","title":"square","content":"pub fn square(\n  p: P5,\n  top_left_x: Float,\n  top_left_y: Float,\n  side_length: Float,\n) -&gt; P5\n A binding to the p5.js [`square`](https://p5js.org/reference/#/p5/square) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.square\np5js_gleam/bindings square","url":"p5js_gleam/bindings.html#square"},{"doc":"p5js_gleam/bindings","title":"start_sketch","content":"pub fn start_sketch(config: SketchConfig(a, b)) -&gt; Nil\n Starts a p5.js sketch with the given configuration.\nSynonyms:\np5js_gleam/bindings.start_sketch\np5js_gleam/bindings start_sketch","url":"p5js_gleam/bindings.html#start_sketch"},{"doc":"p5js_gleam/bindings","title":"stroke","content":"pub fn stroke(p: P5, color_hex: String) -&gt; P5\n A binding to the p5.js [`stroke`](https://p5js.org/reference/#/p5/stroke) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.stroke\np5js_gleam/bindings stroke","url":"p5js_gleam/bindings.html#stroke"},{"doc":"p5js_gleam/bindings","title":"stroke_weight","content":"pub fn stroke_weight(p: P5, weight: Int) -&gt; P5\n A binding to the p5.js [`strokeWeight`](https://p5js.org/reference/#/p5/strokeWeight) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.stroke_weight\np5js_gleam/bindings stroke_weight","url":"p5js_gleam/bindings.html#stroke_weight"},{"doc":"p5js_gleam/bindings","title":"text","content":"pub fn text(\n  p: P5,\n  text: String,\n  bottom_corner_x: Float,\n  bottom_corner_y: Float,\n) -&gt; P5\n A binding to the p5.js [`text`](https://p5js.org/reference/#/p5/text) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.text\np5js_gleam/bindings text","url":"p5js_gleam/bindings.html#text"},{"doc":"p5js_gleam/bindings","title":"text_font","content":"pub fn text_font(p: P5, font: P5Font) -&gt; P5\n A binding to the p5.js [`textFont`](https://p5js.org/reference/#/p5/textFont) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.text_font\np5js_gleam/bindings text_font","url":"p5js_gleam/bindings.html#text_font"},{"doc":"p5js_gleam/bindings","title":"text_font_from_string","content":"pub fn text_font_from_string(p: P5, font: String) -&gt; P5\n A binding to the p5.js [`textFont`](https://p5js.org/reference/#/p5/textFont) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.text_font_from_string\np5js_gleam/bindings text_font_from_string","url":"p5js_gleam/bindings.html#text_font_from_string"},{"doc":"p5js_gleam/bindings","title":"text_size","content":"pub fn text_size(p: P5, size: Int) -&gt; P5\n A binding to the p5.js [`textSize`](https://p5js.org/reference/#/p5/textSize) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.text_size\np5js_gleam/bindings text_size","url":"p5js_gleam/bindings.html#text_size"},{"doc":"p5js_gleam/bindings","title":"text_width","content":"pub fn text_width(p: P5, text: String) -&gt; Float\n A binding to the p5.js [`textWidth`](https://p5js.org/reference/#/p5/textWidth) function. Takes a p5 instance and the function&#39;s arguments and returns the p5 instance.\nSynonyms:\np5js_gleam/bindings.text_width\np5js_gleam/bindings text_width","url":"p5js_gleam/bindings.html#text_width"},{"doc":"p5js_gleam/bindings","title":"p5js_gleam/bindings","content":"","url":"p5js_gleam/bindings.html"}]);